/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import useSyntaxHighlighter from '../../hooks/useSyntaxHighlighter'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

function transformThemeToStyles(theme: ThemeRegistration) {
  const defaultBackground = theme.colors?.['editor.background']
  const foreground = theme.colors?.['editor.foreground']

  console.log('[SyntaxHighlighter] Theme colors:', {
    background: defaultBackground,
    foreground,
    allColors: theme.colors,
  })

  const styles = StyleSheet.create({
    scrollView: {
      flex: 1,
      minHeight: 20,
    },
    codeContainer: {
      padding: 16,
      minWidth: '100%',
    },
    lineContainer: {
      flexDirection: 'row',
      flexWrap: 'wrap',
    },
    baseToken: {
      fontFamily: monospaceFont,
    },
  })

  const colorStyles = new Map<string, TextStyle>()

  return {
    base: styles,
    background: defaultBackground,
    getTokenStyle: (token: ThemedToken): TextStyle => {
      const key = `${token.color || ''}-${token.fontStyle || ''}`

      if (!colorStyles.has(key)) {
        const style: TextStyle = {
          color: token.color || foreground,
          fontStyle: (token.fontStyle ? 'italic' : 'normal') as 'italic' | 'normal',
          fontWeight: token.fontStyle ? 'bold' : 'normal',
        }
        console.log('[SyntaxHighlighter] Created new token style:', { key, token, style })
        colorStyles.set(key, style)
      }

      return colorStyles.get(key)!
    },
  }
}

interface Props extends SyntaxHighlighterProps {
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const SyntaxHighlighter: React.FC<Props> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const result = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  console.log('[SyntaxHighlighter] Highlighted tokens:', result.tokens)

  const styles = useMemo(() => transformThemeToStyles(theme as ThemeRegistration), [theme])

  // Validate tokens array
  const hasValidTokens = result.tokens?.length > 0 && result.tokens.every(line => Array.isArray(line) && line.length > 0)

  if (!hasValidTokens) {
    console.warn('[SyntaxHighlighter] Invalid tokens, falling back to plain text:', {
      tokenCount: result.tokens?.length,
      firstLineTokens: result.tokens?.[0],
    })
    return (
      <ScrollView
        horizontal
        {...scrollViewProps}
        style={[styles.base.scrollView, scrollViewProps?.style]}
        contentContainerStyle={[styles.base.codeContainer, { backgroundColor: theme?.colors?.['editor.background'] || '#1e1e1e' }, scrollViewProps?.contentContainerStyle, containerStyle]}
      >
        <View onStartShouldSetResponder={() => true}>
          <View style={styles.base.lineContainer}>
            <Text style={[styles.base.baseToken, { fontSize, color: theme?.colors?.['editor.foreground'] }, textStyle]}>{text}</Text>
          </View>
        </View>
      </ScrollView>
    )
  }

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[styles.base.scrollView, scrollViewProps?.style]}
      contentContainerStyle={[styles.base.codeContainer, { backgroundColor: theme?.colors?.['editor.background'] || '#1e1e1e' }, scrollViewProps?.contentContainerStyle, containerStyle]}
    >
      <View onStartShouldSetResponder={() => true}>
        {result.tokens.map((line, rowIndex) => (
          <View key={rowIndex} style={[styles.base.lineContainer, { backgroundColor: theme?.colors?.['editor.background'] || '#1e1e1e' }]}>
            {line.map((token, tokenIndex) => {
              const tokenStyle = styles.getTokenStyle(token)
              return (
                <Text key={`${rowIndex}-${tokenIndex}`} style={[styles.base.baseToken, tokenStyle, { fontSize }, textStyle]}>
                  {token.content}
                </Text>
              )
            })}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
