/* eslint-disable no-console */
import type { ThemedToken } from '@shikijs/types'
import type { SyntaxHighlighterProps } from '../types/shiki'
import { useEffect, useState } from 'react'
import { createHighlighter } from '../syntax/highlighter'

export default function useSyntaxHighlighter({ text, language, languageId, theme }: SyntaxHighlighterProps) {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string>()
  const [isLoading, setIsLoading] = useState(true)

  useEffect(() => {
    let mounted = true
    setIsLoading(true)

    async function highlight() {
      if (!text) {
        setIsLoading(false)
        setTokens([])
        return
      }

      try {
        const langs = Array.isArray(language) ? language : [language]
        const highlighter = await createHighlighter({
          langs,
          themes: [theme],
        })

        // Get first language if languageId not provided
        const firstLang = langs[0]
        const langId = languageId || (typeof firstLang === 'string' ? firstLang : typeof firstLang === 'object' && 'name' in firstLang ? firstLang.name : 'text')
        console.log('[Highlighter] Using language:', langId)

        const result = highlighter.codeToTokens(text, {
          lang: langId,
          theme,
        })

        if (mounted && result.tokens) {
          console.log('[Highlighter] Generated tokens:', result.tokens.length)
          setTokens(result.tokens)
          setError(undefined)
        }
      }
      catch (err) {
        console.error('[Highlighter] Error:', err)
        if (mounted) {
          setError(err instanceof Error ? err.message : 'Syntax highlighting failed')
        }
      }
      finally {
        if (mounted) {
          setIsLoading(false)
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  return { tokens, error, isLoading }
}
